import copy
import json
import pytest
import jsonschema
from inventory_provider.routes import msr
from inventory_provider.routes.data import ROUTER_INTERFACES_SCHEMA

DEFAULT_REQUEST_HEADERS = {
    "Content-type": "application/json",
    "Accept": ["application/json"]
}


def test_router_interfaces(juniper_router, client):
    rv = client.get(
        "/data/interfaces/" + juniper_router,
        headers=DEFAULT_REQUEST_HEADERS)

    assert rv.status_code == 200
    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, ROUTER_INTERFACES_SCHEMA)
    assert response  # at least shouldn't be empty


def test_snmp_ids(juniper_router, client):

    snmp_id_list_schema = {
        "$schema": "http://json-schema.org/draft-07/schema#",
        "type": "array",
        "items": {
            "type": "object",
            "properties": {
                "index": {"type": "integer"},
                "name": {"type": "string"},
                "community": {"type": "string"}
            },
            "required": ["index", "name"],
            "additionalProperties": False
        }
    }

    rv = client.get(
        "/testing/snmp/" + juniper_router,
        headers=DEFAULT_REQUEST_HEADERS)

    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, snmp_id_list_schema)
    assert response  # at least shouldn't be empty


def test_router_bgp_routes(juniper_router, client):

    def _no_peering_data_expected(h):
        if h == 'mx2.bru.be.geant.net':
            return None
        return h.startswith('qfx')

    if _no_peering_data_expected(juniper_router):
        pytest.skip('%s is not expected to have bgp peers' % juniper_router)
        return

    rv = client.get(
        "/testing/bgp/" + juniper_router,
        headers=DEFAULT_REQUEST_HEADERS)

    internal_peering_list_schema = copy.deepcopy(msr.PEERING_LIST_SCHEMA)
    internal_peering_list_schema[
        'definitions']['peering-instance']['required'].remove('hostname')

    assert rv.status_code == 200
    response = json.loads(rv.data.decode("utf-8"))
    jsonschema.validate(response, internal_peering_list_schema)
    assert response  # at least shouldn't be empty
